Package: Rcwl
Title: An R interface to the Common Workflow Language
Version: 1.28.0
Authors@R: c(person("Qiang", "Hu", email = "qiang.hu@roswellpark.org", role = c("aut", "cre")),
	     person("Qian", "Liu", email = "qian.liu@roswellpark.org", role = c("aut")))
Description: The Common Workflow Language (CWL) is an open standard for development of data analysis workflows that is portable and scalable across different tools and working environments. Rcwl provides a simple way to wrap command line tools and build CWL data analysis pipelines programmatically within R. It increases the ease of usage, development, and maintenance of CWL pipelines.
Depends: R (>= 3.6), yaml, methods, S4Vectors
Imports: utils, stats, BiocParallel, batchtools, DiagrammeR, shiny,
        R.utils, codetools, basilisk
License: GPL-2 | file LICENSE
Encoding: UTF-8
Suggests: testthat, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
RoxygenNote: 7.3.1
biocViews: Software, WorkflowStep, ImmunoOncology
StagedInstall: no
Config/Bioconductor/UnsupportedPlatforms: windows
git_url: https://git.bioconductor.org/packages/Rcwl
git_branch: RELEASE_3_23
git_last_commit: d4c58a5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:08:54 UTC; biocbuild
Author: Qiang Hu [aut, cre],
  Qian Liu [aut]
Maintainer: Qiang Hu <qiang.hu@roswellpark.org>
