#' Rcwl-package
#'
#' An R package to wrap command line tools and build pipelines with
#' Common Workflow Language (CWL).
#' _PACKAGE
#' @name Rcwl-package
#' @rdname Rcwl-package
#' @aliases Rcwl Rcwl,Rcwl-package
#' @seealso \code{\link{cwlProcess}}
#' @seealso \code{\link{cwlWorkflow}}
#' @seealso \code{\link{cwlStep}} 
#' @seealso \code{\link{runCWL}}
NULL
