% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_cwltool.R
\name{install_cwltool}
\alias{install_cwltool}
\title{install cwltool}
\usage{
install_cwltool()
}
\description{
To download and install cwltool using basilisk
}
